# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

SETUP_PY_BASE = '''
from setuptools import setup

VERSION = '{version}'
BASE_CVS_URL = '{cvs_url}'

setup(
    name='{name}',
    packages=['{module}', ],
    version=VERSION,
    author='{author_name}',
    author_email='{author_email}',
    install_requires=[x.strip() for x in open('{requirements}').readlines()],
    url=BASE_CVS_URL,{download_url}
    test_suite='{tests_module}',
    keywords=[],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Operating System :: OS Independent",{license_classifier}
    ],
)
'''

SETUP_CFG_BASE = '''
[metadata]
description-file = {readme}
'''

MANIFEST_IN_BASE = '''
include *.txt
include {readme}
include {tests_module}/*
'''



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import keyword
import re

from autosetup.compat import urlparse


def validate_module_name(value):
    if not value or not value.strip():
        return False
    value = value.strip()

    return all([
        value.isalnum(),
        value not in keyword.kwlist,
        value[0].isalpha(),
        value not in ('True', 'False', 'None'),
    ])


def validate_name(value):
    if not value or not value.strip():
        return False
    value = value.strip()

    return all([
        re.match('^[\w-]+$', value),
        value[0].isalpha(),
        value not in ('True', 'False', 'None'),
    ])


def validate_email(value):
    if not value or not value.strip():
        return False
    value = value.strip()

    return bool(re.match('^\w+[\w\-\.]+\w+@\w+[\w\-]+\w+\.\w+[\w\-\.]+$', value))  # TODO: beter validation.


def validate_url(value):
    if not value or not value.strip():
        return False
    value = value.strip()

    parsed_url = urlparse(value)
    return bool(parsed_url.netloc) and bool(parsed_url.scheme)



import sys

is_py2 = (sys.version_info[0] == 2)
is_py3 = (sys.version_info[0] == 3)

if is_py2:
    from urlparse import urlparse  # noqa
    input_method = raw_input

if is_py3:
    from urllib.parse import urlparse  # noqa
    input_method = input



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from collections import OrderedDict
import os
import unittest

import mock

from autosetup.main import main


class MainTestCase(unittest.TestCase):
    def setUp(self):
        super(MainTestCase, self).setUp()

        self.arguments = OrderedDict()
        self.arguments['module_name'] = 'supermodule'
        self.arguments['name'] = 'yey'
        self.arguments['version'] = '1.0'
        self.arguments['author_name'] = 'Filipe Waitman1'
        self.arguments['author_email'] = 'filwaitman1@gmail.com'
        self.arguments['cvs_url'] = 'http://bitbucket.org/waitman/supermodule1'
        self.arguments['tests_module'] = 'tests1'
        self.arguments['requirements'] = 'requirements1.txt'
        self.arguments['readme'] = 'README1.rst'
        self.arguments['license'] = 'GPL3'

        self._write_to_file_patched = mock.patch('autosetup.main._write_to_file')
        self.write_to_file_patched = self._write_to_file_patched.start()

        self._input_method_patched = mock.patch('autosetup.utils.input_method')
        self.input_method_patched = self._input_method_patched.start()

        self.args_mock = mock.Mock()
        self.args_mock.configure_mock(
            module_name=None,
            name=None,
            version=None,
            author_name=None,
            author_email=None,
            cvs_url=None,
            tests_module=None,
            requirements=None,
            readme=None,
            license=None,
            use_defaults=False,
            overwrite_all=False,
        )
        self._get_command_args_patched = mock.patch('autosetup.main._get_command_args', return_value=self.args_mock)
        self.get_command_args_patched = self._get_command_args_patched.start()
        self.get_command_args_patched.configure_mock()

    def tearDown(self):
        self._write_to_file_patched.stop()
        self._input_method_patched.stop()
        self._get_command_args_patched.stop()

        super(MainTestCase, self).tearDown()

    def assertFileContents(self, path_name, requirements_path, readme_path):
        current_dir = os.path.dirname(os.path.abspath(__file__))

        setup_cfg = open(os.path.join(current_dir, 'data', path_name, 'setup.cfg')).read()
        self.write_to_file_patched.assert_any_call(mock.ANY, setup_cfg, mock.ANY)

        manifest_in = open(os.path.join(current_dir, 'data', path_name, 'MANIFEST.in')).read()
        self.write_to_file_patched.assert_any_call(mock.ANY, manifest_in, mock.ANY)

        license = open(os.path.join(current_dir, 'data', path_name, 'LICENSE')).read()
        self.write_to_file_patched.assert_any_call(mock.ANY, license, mock.ANY)

        requirements = open(os.path.join(current_dir, 'data', path_name, requirements_path)).read()
        self.write_to_file_patched.assert_any_call(mock.ANY, requirements, mock.ANY)

        readme = open(os.path.join(current_dir, 'data', path_name, readme_path)).read()
        self.write_to_file_patched.assert_any_call(mock.ANY, readme, mock.ANY)

        setup_py = open(os.path.join(current_dir, 'data', path_name, 'setup.py')).read()
        self.write_to_file_patched.assert_any_call(mock.ANY, setup_py, mock.ANY)

    def test_cli_common(self):
        self.input_method_patched.side_effect = self.arguments.values()

        result = main(return_values=True)

        self.assertEquals(result['module'], self.arguments['module_name'])
        self.assertEquals(result['name'], self.arguments['name'])
        self.assertEquals(result['version'], self.arguments['version'])
        self.assertEquals(result['author_name'], self.arguments['author_name'])
        self.assertEquals(result['author_email'], self.arguments['author_email'])
        self.assertEquals(result['cvs_url'], self.arguments['cvs_url'])
        self.assertEquals(result['tests_module'], self.arguments['tests_module'])
        self.assertEquals(result['requirements'], self.arguments['requirements'])
        self.assertEquals(result['readme'], self.arguments['readme'])
        self.assertEquals(result['license'], self.arguments['license'])

        self.assertFileContents('test_cli_common', 'requirements1.txt', 'README1.rst')

    def test_cli_license_other(self):
        arguments = self.arguments.copy()
        arguments['license'] = 'OTHER'
        self.input_method_patched.side_effect = arguments.values()

        result = main(return_values=True)

        self.assertEquals(result['module'], self.arguments['module_name'])
        self.assertEquals(result['name'], self.arguments['name'])
        self.assertEquals(result['version'], self.arguments['version'])
        self.assertEquals(result['author_name'], self.arguments['author_name'])
        self.assertEquals(result['author_email'], self.arguments['author_email'])
        self.assertEquals(result['cvs_url'], self.arguments['cvs_url'])
        self.assertEquals(result['tests_module'], self.arguments['tests_module'])
        self.assertEquals(result['requirements'], self.arguments['requirements'])
        self.assertEquals(result['readme'], self.arguments['readme'])
        self.assertEquals(result['license'], 'OTHER')
        self.assertEquals(result['license_classifier'], '')

        self.assertFileContents('test_cli_license_other', 'requirements1.txt', 'README1.rst')

    def test_cli_defaults(self):
        arguments = self.arguments.copy()
        arguments['name'] = ''
        arguments['version'] = ''
        arguments['tests_module'] = ''
        arguments['requirements'] = ''
        arguments['readme'] = ''
        self.args_mock.use_defaults = True

        self.input_method_patched.side_effect = arguments.values()

        result = main(return_values=True)

        self.assertEquals(result['name'], self.arguments['module_name'])
        self.assertEquals(result['version'], '0.0.1')
        self.assertEquals(result['tests_module'], 'tests')
        self.assertEquals(result['requirements'], 'requirements.txt')
        self.assertEquals(result['readme'], 'README.rst')

        self.assertEquals(result['module'], self.arguments['module_name'])
        self.assertEquals(result['author_name'], self.arguments['author_name'])
        self.assertEquals(result['author_email'], self.arguments['author_email'])
        self.assertEquals(result['cvs_url'], self.arguments['cvs_url'])
        self.assertEquals(result['license'], self.arguments['license'])

        self.assertFileContents('test_cli_defaults', 'requirements.txt', 'README.rst')

    def test_cli_params_being_passed(self):
        args_mock = mock.Mock()
        args_mock.configure_mock(
            module_name='supersuper',
            name='namename',
            version='42',
            author_name='Hamster Smith',
            author_email='hamster@smith.com',
            cvs_url='http://www.google.com',
            tests_module='whatever',
            requirements='requires.txt',
            readme='RTFM',
            license='GPL3',
            use_defaults=False,
            overwrite_all=False,
        )
        self.get_command_args_patched.return_value = args_mock

        result = main(return_values=True)

        self.assertEquals(result['module'], 'supersuper')
        self.assertEquals(result['name'], 'namename')
        self.assertEquals(result['version'], '42')
        self.assertEquals(result['author_name'], 'Hamster Smith')
        self.assertEquals(result['author_email'], 'hamster@smith.com')
        self.assertEquals(result['cvs_url'], 'http://www.google.com')
        self.assertEquals(result['tests_module'], 'whatever')
        self.assertEquals(result['requirements'], 'requires.txt')
        self.assertEquals(result['readme'], 'RTFM')
        self.assertEquals(result['license'], 'GPL3')

        self.assertFileContents('test_cli_params_being_passed', 'requires.txt', 'RTFM')

    def test_main_params_being_passed(self):
        result = main(
            module_name='supersuper',
            name='namename',
            version='42',
            author_name='Hamster Smith',
            author_email='hamster@smith.com',
            cvs_url='http://www.google.com',
            tests_module='whatever',
            requirements='requires.txt',
            readme='RTFM',
            license='GPL3',
            return_values=True
        )

        self.assertEquals(result['module'], 'supersuper')
        self.assertEquals(result['name'], 'namename')
        self.assertEquals(result['version'], '42')
        self.assertEquals(result['author_name'], 'Hamster Smith')
        self.assertEquals(result['author_email'], 'hamster@smith.com')
        self.assertEquals(result['cvs_url'], 'http://www.google.com')
        self.assertEquals(result['tests_module'], 'whatever')
        self.assertEquals(result['requirements'], 'requires.txt')
        self.assertEquals(result['readme'], 'RTFM')
        self.assertEquals(result['license'], 'GPL3')

        self.assertFileContents('test_main_params_being_passed', 'requires.txt', 'RTFM')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from datetime import date
import os

from autosetup import licenses, templates, validators
from autosetup.utils import _write_to_file, _get_value, _first_set, _get_command_args

_LICENSE_CONTENT_MAPPING = {
    'MIT': licenses.MIT,
    'GPL2': licenses.GPL2,
    'GPL3': licenses.GPL3,
    'OTHER': '',
}

_LICENSE_CLASSIFIER_MAPPING = {
    'MIT': '\n        "License :: OSI Approved :: MIT License",',
    'GPL2': '\n        "License :: OSI Approved :: GNU General Public License (GPL)",',
    'GPL3': '\n        "License :: OSI Approved :: GNU General Public License (GPL)",',
    'OTHER': '',
}


def main(
    module_name=None, name=None, version=None, author_name=None, author_email=None,
    cvs_url=None, tests_module=None, requirements=None, readme=None, license=None,
    use_defaults=None, overwrite_all=None, return_values=False

):
    cwd = os.getcwd()
    args = _get_command_args()

    use_defaults = _first_set(use_defaults, args.use_defaults, False)
    overwrite_all = _first_set(overwrite_all, args.overwrite_all, False)

    module_name_initial = _first_set(module_name, args.module_name)
    module_name = _get_value('Module name', initial=module_name_initial, validator=validators.validate_module_name)

    name_initial = _first_set(name, args.name, module_name if use_defaults else None)
    version_initial = _first_set(version, args.version, '0.0.1' if use_defaults else None)
    author_name_initial = _first_set(author_name, args.author_name)
    author_email_initial = _first_set(author_email, args.author_email)
    cvs_url_initial = _first_set(cvs_url, args.cvs_url)
    tests_module_initial = _first_set(tests_module, args.tests_module, 'tests' if use_defaults else None)
    requirements_initial = _first_set(requirements, args.requirements, 'requirements.txt' if use_defaults else None)
    readme_initial = _first_set(readme, args.readme, 'README.rst' if use_defaults else None)
    license_initial = _first_set(license, args.license)

    name = _get_value('Name', initial=name_initial, default=module_name, validator=validators.validate_name)
    version = _get_value('Initial version', initial=version_initial, default='0.0.1')
    author_name = _get_value('Author name', initial=author_name_initial)
    author_email = _get_value('Author email', initial=author_email_initial, validator=validators.validate_email)
    cvs_url = _get_value('CVS (GitHub, Bitbucket) URL', initial=cvs_url_initial, validator=validators.validate_url)
    tests_module = _get_value('Tests module', initial=tests_module_initial, default='tests')
    requirements = _get_value('Requirements file', initial=requirements_initial, default='requirements.txt')
    readme = _get_value('README file', initial=readme_initial, default='README.rst')
    license = _get_value('License', initial=license_initial, choices=('MIT', 'GPL2', 'GPL3', 'OTHER'))

    final_params = dict(
        module=module_name,
        name=name,
        version=version,
        author_name=author_name,
        author_email=author_email,
        cvs_url=cvs_url,
        tests_module=tests_module,
        requirements=requirements,
        readme=readme,
        license=license,

        license_classifier=_LICENSE_CLASSIFIER_MAPPING[license],
        year=date.today().year,
        download_url="\n    download_url='{}/tarball/{}'.format(BASE_CVS_URL, VERSION)," if 'github' in cvs_url else ''
    )

    requirements_content = '\n'
    _write_to_file(os.path.join(cwd, requirements), requirements_content, overwrite_all)

    readme_content = name + '\n' + ('=' * len(name)) + '\n'
    _write_to_file(os.path.join(cwd, readme), readme_content, overwrite_all)

    manifest_content = templates.MANIFEST_IN_BASE.format(**final_params).strip() + '\n'
    _write_to_file(os.path.join(cwd, 'MANIFEST.in'), manifest_content, overwrite_all)

    license_content = _LICENSE_CONTENT_MAPPING[license].format(**final_params)
    _write_to_file(os.path.join(cwd, 'LICENSE'), license_content, overwrite_all)

    setup_cfg_content = templates.SETUP_CFG_BASE.format(**final_params).strip() + '\n'
    _write_to_file(os.path.join(cwd, 'setup.cfg'), setup_cfg_content, overwrite_all)

    setup_py_content = templates.SETUP_PY_BASE.format(**final_params).strip() + '\n'
    _write_to_file(os.path.join(cwd, 'setup.py'), setup_py_content, overwrite_all)

    if return_values:
        return final_params  # For testing purposes.

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import argparse
import os

from autosetup.compat import input_method


def _confirm(message, default=None):
    post_message = '[y/n]'
    default_answer = None

    if default is not None:
        if default is True:
            default_answer = 'Y'
            post_message = '[Y/n]'
        elif default is False:
            default_answer = 'N'
            post_message = '[y/N]'
        else:
            raise RuntimeError

    answer = None
    while answer not in ('Y', 'N'):
        answer = input_method('{} {}: '.format(message, post_message)).upper()
        if not answer and default_answer:
            answer = default_answer

    return answer == 'Y'


def _write_to_file(file_path, content, force):
    if os.path.exists(file_path) and not force:
        confirm = _confirm('Overwrite {}?'.format(file_path), default=False)
        if not confirm:
            return

    with open(file_path, 'w') as f:
        f.write(content)


def _get_value(message, initial=None, required=True, default=None, validator=None, choices=None):
    value = initial
    first_iteration = True

    post_message = ''
    if choices is not None:
        post_message += ' (one of "{}")'.format('", "'.join(choices))
    if default is not None:
        post_message += ' [default: "{}"]'.format(default)
    prompt_message = '{}{}: '.format(message, post_message)

    while True:
        if first_iteration and initial:
            value = initial
            first_iteration = False

        else:
            value = input_method(prompt_message).strip()

        if not value:
            if default is not None:
                value = default
            elif not required:
                value = ''
            else:
                continue

        if validator:
            if validator(value):
                return value
        elif choices:
            if value in choices:
                return value
        else:
            return value


def _first_set(*values):
    for value in values:
        if value is not None:
            return value


def _get_command_args():
    parser = argparse.ArgumentParser(description='autosetup-py')
    parser.add_argument('--module', dest='module_name', help='Module name')
    parser.add_argument('--name', dest='name', help='Name [default: module name]')
    parser.add_argument('--version', dest='version', help='Initial version [default: "0.0.1"]')
    parser.add_argument('--author_name', dest='author_name', help="Author name")
    parser.add_argument('--author_email', dest='author_email', help="Author email")
    parser.add_argument('--cvs_url', dest='cvs_url', help="CVS (GitHub, Bitbucket) URL")
    parser.add_argument('--tests_module', dest='tests_module', help='Tests module [default: "tests"]')
    parser.add_argument('--requirements', dest='requirements', help='Requirements file [default: "requirements.txt"]')
    parser.add_argument('--readme', dest='readme', help='README file [default: "README.rst"]')
    parser.add_argument('--license', dest='license', default='', help='License ("MIT", "GPL2, "GPL3", or "OTHER")')
    parser.add_argument('--use-defaults', dest='use_defaults', action='store_true', default=False)
    parser.add_argument('--overwrite-all', dest='overwrite_all', action='store_true', default=False)
    return parser.parse_args()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import unittest

from autosetup.validators import validate_module_name, validate_name, validate_email, validate_url


class ValidateModuleNameTestCase(unittest.TestCase):
    def test_regular_chars(self):
        self.assertTrue(validate_module_name('potato'))
        self.assertTrue(validate_module_name('CrazyCapitalLETTER'))
        self.assertTrue(validate_module_name('z'))

    def test_numbers(self):
        self.assertTrue(validate_module_name('area51'))
        self.assertTrue(validate_module_name('hero2vilain'))

        self.assertFalse(validate_module_name('42isthwanswer'))
        self.assertFalse(validate_module_name('666'))

    def test_empty(self):
        self.assertFalse(validate_module_name(''))
        self.assertFalse(validate_module_name('     '))

    def test_reserved_keywords(self):
        self.assertFalse(validate_module_name('for'))
        self.assertFalse(validate_module_name('if'))

    def test_special_chars(self):
        self.assertFalse(validate_module_name('money-o-rama'))
        self.assertFalse(validate_module_name('ImThe#1'))
        self.assertFalse(validate_module_name('Supercool!'))

    def test_builtins(self):
        # Although I disagree you actually *can* override those ones.
        self.assertTrue(validate_module_name('AssertionError'))
        self.assertTrue(validate_module_name('zip'))

    def test_restricted_builtins(self):
        self.assertFalse(validate_module_name('True'))
        self.assertFalse(validate_module_name('False'))
        self.assertFalse(validate_module_name('None'))


class ValidateNameTestCase(unittest.TestCase):
    def test_regular_chars(self):
        self.assertTrue(validate_name('potato'))
        self.assertTrue(validate_name('CrazyCapitalLETTER'))
        self.assertTrue(validate_name('z'))

    def test_numbers(self):
        self.assertTrue(validate_name('area51'))
        self.assertTrue(validate_name('hero2vilain'))

        self.assertFalse(validate_name('42isthwanswer'))
        self.assertFalse(validate_name('666'))

    def test_empty(self):
        self.assertFalse(validate_name(''))
        self.assertFalse(validate_name('     '))

    def test_reserved_keywords(self):
        self.assertTrue(validate_name('for'))  # No reserved words since this is just the name.
        self.assertTrue(validate_name('if'))  # No reserved words since this is just the name.

    def test_special_chars(self):
        self.assertTrue(validate_name('money-o-rama'))

        self.assertFalse(validate_name('ImThe#1'))
        self.assertFalse(validate_name('Supercool!'))

    def test_builtins(self):
        self.assertTrue(validate_name('AssertionError'))
        self.assertTrue(validate_name('zip'))

    def test_restricted_builtins(self):
        self.assertFalse(validate_name('True'))
        self.assertFalse(validate_name('False'))
        self.assertFalse(validate_name('None'))


class ValidateEmailTestCase(unittest.TestCase):
    def test_common(self):
        self.assertTrue(validate_email('hamster@sloth.com'))
        self.assertTrue(validate_email('HAMSter@sloth.co.nz'))
        self.assertTrue(validate_email('hamster1995x0x0@sloth.com'))
        self.assertTrue(validate_email('hams.ter@sloth.com'))
        self.assertTrue(validate_email('hamster-king@sloth.com'))

    def test_empty(self):
        self.assertFalse(validate_email(''))
        self.assertFalse(validate_email('     '))

    def test_malformed_emails(self):
        self.assertFalse(validate_email('hamsterslothcom'))
        self.assertFalse(validate_email('hamster@slothcom'))
        self.assertFalse(validate_email('hamster@sloth'))
        self.assertFalse(validate_email('hamster@'))
        self.assertFalse(validate_email('@sloth.com'))
        self.assertFalse(validate_email('hamster'))
        self.assertFalse(validate_email('hamster@@sloth.com'))
        self.assertFalse(validate_email('hamster@sloth..com'))
        self.assertFalse(validate_email('hamster.@sloth.com'))
        self.assertFalse(validate_email('hamster@.sloth.com'))
        self.assertFalse(validate_email('hamster@sloth.-com'))
        self.assertFalse(validate_email('hamster-@sloth.com'))
        self.assertFalse(validate_email('hamster@-sloth.com'))


class ValidateUrlTestCase(unittest.TestCase):
    def test_common(self):
        self.assertTrue(validate_url('http://www.google.com'))
        self.assertTrue(validate_url('https://youtube.com/something/else'))

    def test_empty(self):
        self.assertFalse(validate_url(''))
        self.assertFalse(validate_url('     '))

    def test_malformed_emails(self):
        self.assertFalse(validate_url('hamster'))
        self.assertFalse(validate_url('hamster.com'))
        self.assertFalse(validate_url('http'))
        self.assertFalse(validate_url('http://'))



from setuptools import setup

VERSION = '0.0.2'
BASE_CVS_URL = 'http://github.com/filwaitman/autosetup.py'

setup(
    name='autosetup-py',
    packages=['autosetup', ],
    version=VERSION,
    author='Filipe Waitman',
    author_email='filwaitman@gmail.com',
    install_requires=[x.strip() for x in open('requirements.txt').readlines()],
    url=BASE_CVS_URL,
    download_url='{}/tarball/{}'.format(BASE_CVS_URL, VERSION),
    test_suite='tests',
    keywords=[],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={
        'console_scripts': [
            'autosetup-py=autosetup.main:main'
        ]
    }
)
